#!/bin/bash 

# install.sh
# Install EMG patches log results for LMI and SiteProtector
#

#set -x

SUCCESS="0"
NOT_FOUND="NOT_FOUND";
PATCH_CONF="/etc/crm/patches.config"
DEFAULT_PATCH_ARCHIVE_DIR=/etc/lmi/patches

# Read patch.conf to get value corresponding to input name
# - for example:  GetConfigValue "PatchArchiveDir=" PATCH_DIR ==> sets PATCH_DIR to "/etc/lmi/patches" 
# - returns NOT_FOUND if no corresponding key is found
GetConfigValue()
{
	KEY="$1"
	if [ -f $PATCH_CONF ] ; then
		VALUE=`cat $PATCH_CONF | sed -n "/$KEY/p" | sed "s/$KEY//"`
	fi 
	if [ x"" == x"$VALUE" ] ; then
		VALUE=$NOT_FOUND
	fi
	eval "$2=$VALUE"
}

# Determine patch ID (*.info) file in patch contents in current dir
GetInfoFilename()
{
	NUM_FILE=`find . -maxdepth 1 -regex '^\./patch.*\.info' -print | wc -l`
	if [ x"1" != x"$NUM_FILE" ] ; then
		echo "More than one *.info file exists in current directory---cannot determine patch ID."
		echo "Please extract patch to empty directory and try again."
		exit 1;
	else
		INFO=`find . -maxdepth 1 -regex '^\./patch.*\.info' -print`
	fi
	eval "$1=$INFO"
}

# Determine patch ID (*.info) file in patch contents in current dir
GetPatchID()
{
	GetInfoFilename INFO_FILE 
	# INFO_FILE should be something like ./patch285.info
	ID=`echo "$INFO_FILE" | sed s/[^0-9]//g`
	if [ x"" == x"$ID" ] ; then
		echo "Unable to parse patch number from *.info file:  $INFO_FILE.  Exiting."
		exit 1;
	fi
	eval "$1=$ID"
}

GetPatchID PATCH_ID
STATUS_WARN=0
STATUS_ERR=0

MODE="install"
FIPS_MODE=0

Log() { echo $* ; }

LogError()
{
    Log ERROR: $*
    STATUS_ERR=1
}

LogWarning()
{
    Log WARNING: $*
    STATUS_WARN=1
}

# ProvG Helper Functions
KillDaemon()
{
  if ps -e | grep issDaemon >/dev/null 2>&1; then
    Log "Stopping ISS Daemon."
    /etc/init.d/issDaemon stop >/dev/null 2>&1
  fi
}


Install()
{

    KillDaemon

    INSTALL_LIST=
    for i in $@ ; do
        PKGVER=`rpm -qp $i`
        BASE=`basename $i -new.rpm`
        if rpm -q $BASE >/dev/null 2>&1 ; then
            INSTVER=`rpm -q $BASE`
            #if iss-rpm-compare $i ; then
                Log Upgrading $INSTVER to $PKGVER
                if [ -f $BASE-new.sigtgz ] ; then 
                    tar -C / -xzf $BASE-new.sigtgz || LogError Failed to extract signatures for $BASE
                fi
                INSTALL_LIST="$i $INSTALL_LIST"
            #else
            #    LogWarning Skipping $PKGVER, $INSTVER already installed
            #fi
        else
            Log Installing $PKGVER
            if [ -f $BASE-new.sigtgz ] ; then
                tar -C / -xzf $BASE-new.sigtgz || LogError Failed to extract signatures for $BASE
            fi
            INSTALL_LIST="$i $INSTALL_LIST"
        fi
    done
    if [ ! -z "$INSTALL_LIST" ] ; then
        rpm $RPM_INSTALL_OPTS $INSTALL_LIST && return 0 || LogError Failed while installing packages $INSTALL_LIST
    fi
    return 1
}

Uninstall()
{

    KillDaemon

    INSTALL_LIST=
    for i in $@ ; do
        PKGVER=`rpm -qp $i`
        BASE=`basename $i -original.rpm`
        if rpm -q $BASE >/dev/null 2>&1 ; then
            INSTVER=`rpm -q $BASE`
            #if iss-rpm-compare $i ; then
                Log Downgrading $INSTVER to $PKGVER
                if [ -f $BASE-original.sigtgz ] ; then
                    tar -C / -xzf $BASE-original.sigtgz || LogError Failed to extract signatures for $BASE
                fi
                INSTALL_LIST="$i $INSTALL_LIST"
            #else
                #LogWarning Skipping $PKGVER, $INSTVER already installed
            #fi
        else
            Log Installing $PKGVER
            if [ -f $BASE-original.sigtgz ] ; then
                tar -C / -xzf $BASE-original.sigtgz || LogError Failed to extract signatures for $BASE
            fi
            INSTALL_LIST="$i $INSTALL_LIST"
        fi
    done
    if [ ! -z "$INSTALL_LIST" ] ; then
        rpm $RPM_UNINSTALL_OPTS $INSTALL_LIST && return 0 || LogError Failed while installing packages $INSTALL_LIST
    fi
    return 1
}

CondInstall()
{
    INSTALL_LIST=
    for i in $@ ; do
        BASE=`basename $i .rpm`
        if rpm -q $BASE >/dev/null 2>&1 ; then
            INSTALL_LIST="$i $INSTALL_LIST"
        else
            Log Skipping $i, conditional package was not installed
        fi
    done
    Install $INSTALL_LIST
    return $?
}

RunScript()
{
    if [ -f "$1" ]; then
        chmod +x $1
        ./${1} || LogError script $1 failed
        Log Script $1 completed
    else
        LogError Missing script $1
        return $?
    fi
}

TestForFIPSMode()
{
    local fipsMode=`service issFIPS status | grep -i unused`
    if [ -z "$fipsMode" ]; then
        FIPS_MODE=1
    fi
}

Log Hotfix Installation Started
echo ""

RPM_INSTALL_OPTS="--force -Uvh"

RPM_UNINSTALL_OPTS="--force -Uvh"

# BEGIN CUSTOMIZATION SECTION
ShowUsage()
{
	Log "install.sh [OPTIONS]"
	Log "	-r Rollback to previous settings."
	Log "	-h Show this message."
}

while getopts "rh" cmdopt $*
do
	case "$cmdopt" in
		"h")
		    MODE="usage"
		    break
		    ;;
		"r")
		    MODE="rollback"
		    break
		    ;;
		"*")
		    MODE="badargs"
		    break
		    ;;
	esac
done

#transform xml - need to supply 2 arguments
#1st arg - XML to transform
#2nd arg - transform to apply
TransformXML()
{
        #echo $#
	TEMP_XML=/tmp/tmp`basename $1`
	#backup target
	cp -f $1 $1.bak
	#do transform
	Xalan -o $TEMP_XML $1 $2 
	#evaluate return code
	if [ $? -eq 0 ]; then
		#test output
		eval $(echo `Xalan "$TEMP_XML" "$VALIDATOR" | awk '/The regex is/{regex=$4} END{print "export regex=\""regex"\""}'`)
		if [ "$2" == "$UPDATER" ] && [ "$regex" == "$NEWEXPRESSION" ]; then
			cp -f $TEMP_XML $1
			rm -f $TEMP_XML
		elif [ "$2" == "$REVERTER" ] && [ "$regex" == "$OLDEXPRESSION" ]; then
			cp -f $TEMP_XML $1
			rm -f $TEMP_XML
		else
			LogError Failed to read correct values from translated xml.
			cp -f $1.bak $1
		fi
	else
		LogError Failed to translate xml.
	fi
}


if [ $MODE == "usage" ]; then
	ShowUsage
	echo "No archiving of patch log due to usage argument."
	exit 1
else
	eval $(echo `issasApache -a si | awk ' /Last Firmware Update/{firm=$7} /Model Number/{model=substr($2,10)} END{print "export fwversion="firm" model="model} '`)

	echo $fwversion $model

	TestForFIPSMode

	if [ $FIPS_MODE -ne 0 ]; then
		echo "Appliance is running in FIPS mode."
		echo "Patch cannot be applied. Please contact IBM Support."
		exit 1
	fi

	echo "Checking Firmware Version..."
	if [ -n "$fwversion" ]; then
		if [ $fwversion == 4.6.2 ]; then
			xputmp=`issasApache -a si | grep "Intrusion Prevention" | grep version`
			xpuversion=`expr match "$xputmp" '.*\([0-9][0-9]\.[0-9][0-9]*\?[0-9]\)'`
			if [ $xpuversion \< "33.120" ]
			then
				echo "XPU version 33.120 or higher required. Update XPU and try again. If current XPU is 33.120 or higher, then contact support."
                                exit 1 
			else
			
			service issDaemon stop
			service iss-watchdog stop
			service iss-lum stop
			service apache2-lmi stop

			#install common x32 libs first
			echo "Installing non-model specific modules"
                        if [ $MODE == "install" ]; then
                                #Install new rpms
				Install iss-watchdog-new.rpm
				Install iss-csf-new.rpm
				Install iss-glmi-new.rpm
				Install iss-ppd-new.rpm
				Install issMSL-new.rpm
				Install iss-proventiag-spa-new.rpm
				Install iss-log-utils-new.rpm
				Install iss-provg-tools-new.rpm
				Install iss-proventia-setup-new.rpm
				Install iss-unified-lmi-new.rpm
			elif [ $MODE == "rollback" ]; then
                                #Uninstall new rpms
				Uninstall iss-watchdog-original.rpm
				Uninstall iss-csf-original.rpm
				Uninstall iss-glmi-original.rpm
				Uninstall iss-ppd-original.rpm
				Uninstall issMSL-original.rpm
				Uninstall iss-proventiag-spa-original.rpm
				Uninstall iss-log-utils-original.rpm
				Uninstall iss-provg-tools-original.rpm
				Uninstall iss-proventia-setup-original.rpm
				Uninstall iss-unified-lmi-original.rpm 
			fi

                        #check model go to directory
                        if [[ $model =~ "GX7.*" ]]; then
                                echo "Entering gx7 direcoty"
                                cd gx7
                                #install pktlog rpm x64 only on gx7.
                                #if [ $MODE == "install" ]; then
                                #current change is gx7 only for secmgr
                                #elif [ $MODE == "rollback" ]; then
                                #fi
                        elif [[ $model =~ "GX6.*" ]]; then
                                echo "Entering gx6 direcoty"
                                cd gx6
			 	#pktlog is common between all non-gx7 models.
				#msglibs only used in gx6	
                                #if [ $MODE == "install" ]; then
				#elif [ $MODE == "rollback" ]; then
				#fi
                        #else
                                #pktlog is common between all non-gx7 models.
                                #if [ $MODE == "install" ]; then
				#elif [ $MODE == "rollback" ]; then
				#fi
                        fi

			#check mode
                        if [ $MODE == "install" ]; then
                                #Install new rpms
				Install iss-netengine-new.rpm
                                Install iss-secmgr-new.rpm
				Install iss-proventiag-crm-new.rpm
				Install iss-snep-new.rpm

                        elif [ $MODE == "rollback" ]; then
                                Uninstall iss-netengine-original.rpm
                                Uninstall iss-secmgr-original.rpm
				Uninstall iss-proventiag-crm-original.rpm
				Uninstall iss-snep-original.rpm
			fi
	
			#common x32 npu rpm on gx6 only
			#move back one dir for if gx7 for patchinfo file
        		if [[ $model =~ "GX7.*" ]]; then
				echo "Leaving gx7 directory"
				cd ..
                        fi

			#need to go back for the patch info file install
                        if [[ $model =~ "GX6.*" ]]; then
				echo "Leaving gx6 directory"
                                cd ..
                        fi
		
			service apache2-lmi start
			service iss-lum start	
			service iss-watchdog start
			service issDaemon start

			cc=`service iss-spa status | grep running`
                        if [ -n "$cc" ]
                        then
                                service iss-spa restart
                        fi

			fi
		else
			echo "Incorrect firmware version for this patch. Firmware 4.6.2 is required. Your firmware version is $fwversion."
			exit 1
		fi
	else
			echo "Please ensure the issDaemon is running prior to installing this patch."
                	exit 1
	fi
fi

#END CUSTOMIZATION SECTION
echo ""
if [ $STATUS_ERR -ne 0 ] ; then
	Log "result_code = failed"
	Log "Hotfix Installation Failed"
	exit 1
elif [ $STATUS_WARN -ne 0 ] ; then
	Log "result_code = warning"
	Log "Hotfix installation completed with warnings"
else
	Log "result_code = success"
	Log "Hotfix Installation Completed Successfully"
fi

if [ -n "$fwversion" ] && [ $fwversion == 4.6.2 ]; then
	GetConfigValue "PatchArchiveDir=" PATCH_ARCHIVE_DIR
	if [ x"$NOT_FOUND" == x"$PATCH_ARCHIVE_DIR" ] ; then
		PATCH_ARCHIVE_DIR=$DEFAULT_PATCH_ARCHIVE_DIR
	fi

	# Test for existence of PATCH_ARCHIVE_DIR (create if doesn't exist)
	mkdir -p $PATCH_ARCHIVE_DIR
	if [ "$?" != "$SUCCESS" ] ; then
		echo "Unable to create $PATCH_ARCHIVE---exiting."
		exit $?
	fi

	TIMESTAMP=$(date +"%Y-%m-%d %H:%M:%S")

	# If archived results for patch already exists, append new results 
	# to bottom of file
	ACTION="unknown action"
	if [ "install" == "$MODE" ] ; then
		ACTION="Installed"
	elif [ "rollback" == "$MODE" ] ; then
		ACTION="Rolled Back"
	fi
	echo "$ACTION on $TIMESTAMP for FW $FW_VERSION" >> $PATCH_ARCHIVE_DIR/patch$PATCH_ID.results

	# Copy Patch's .info file to patch archive
	GetInfoFilename PATCH_INFO_FILE
	cp -u $PATCH_INFO_FILE $PATCH_ARCHIVE_DIR

fi
exit $SUCCESS
