#!/bin/bash

KEYPATH=/etc/crm/ha
SSLPATH=/opt/iss/etc/ssl
CERTPATH=${SSLPATH}/ha
HOST=`cat /etc/HOSTNAME`
APACHECAPATH=/var/www/lmi/public/ca
OPENSSLCLI=/opt/iss/bin/openssl
NUMPARAMS=0
APACHECONF=/etc/apache2/include_lmi_ha.conf


CheckUsage()
{
    if [ $# -lt 1 -o $# -gt $NUMPARAMS ]
    then
        Usage
        exit 1
    fi
}

Usage()
{
    name=`basename $0`
    echo "Usage: $name < -k passphrase | -d | -c IPaddress | -h IPaddress | -m IPaddress | -r >"
    echo "  -k : create Keys"
    echo "  -d : Delete keys"
    echo "  -c : Configure web server for HA setup"
    echo "  -h : Configure web server for HA operation"
    echo "  -m : Configure web server during HA manual configuration"
    echo "  -r : Reset web server configuration"
    echo
}

CreateKeys()
{
    test -d $KEYPATH || mkdir -p $KEYPATH
    test -d $CERTPATH || mkdir -p $CERTPATH

    # Test if default openssl.cnf is installed.
# should a custom openssl.cnf be used? see certpath and cert file names
    if [ ! -f $SSLPATH/openssl.cnf ]
    then
        echo Cannot find $SSLPATH/openssl.cnf
        exit 1
    fi

    # Create the CA certificate.
    if [ ! -f $CERTPATH/CAcrt.pem ]
    then
        $OPENSSLCLI genrsa -rand /proc/apm:/proc/cpuinfo:/proc/dma:/proc/filesystems:/proc/interrupts:/proc/ioports:/proc/pci:/proc/rtc:/proc/uptime -out $CERTPATH/CAkey.pem 2048
        cat << EOF | $OPENSSLCLI req -new -key $CERTPATH/CAkey.pem -out $CERTPATH/CA.csr -sha256
XY
Any State
Any Town
Any Organization
Any Unit
$HOST
postmaster@localhost


EOF

        $OPENSSLCLI x509 -req -days 3650 -in $CERTPATH/CA.csr -out $CERTPATH/CAcrt.pem -signkey $CERTPATH/CAkey.pem -sha256
   	    # Create PKCS12 format file for CA cert/key
        $OPENSSLCLI pkcs12 -name ibm.security.ha -export -in $CERTPATH/CAcrt.pem -inkey $CERTPATH/CAkey.pem -out $CERTPATH/ibm.security.ha.p12 -descert -password pass:ibmss01
        
        # Add HA PKCS12 to keysotre
        # /etc/crm/manage_keystore.sh -p /var/spool/crm/leafcerts/iss-working.kdb $CERTPATH/ibm.security.ha.p12
    fi

    # Create a client certificate (des3 is required by MSL code)
    # required for mutual authentication with the web server.
    # Sign the certificate request using the CA created above.
    if [ -f $CERTPATH/CAcrt.pem -a ! -f $KEYPATH/clientcrt.pem ]
    then
        #$OPENSSLCLI genrsa -rand /proc/apm:/proc/cpuinfo:/proc/dma:/proc/filesystems:/proc/interrupts:/proc/ioports:/proc/pci:/proc/rtc:/proc/uptime -des3 -passout pass:$PASSWORD -out $KEYPATH/clientkey.pem 1536
        $OPENSSLCLI genrsa -rand /proc/apm:/proc/cpuinfo:/proc/dma:/proc/filesystems:/proc/interrupts:/proc/ioports:/proc/pci:/proc/rtc:/proc/uptime -out $KEYPATH/clientkey.pem 2048

#        cat << EOF | $OPENSSLCLI req -new -passin pass:$PASSWORD -key $KEYPATH/clientkey.pem -out $KEYPATH/client.csr
        cat << EOF | $OPENSSLCLI req -new -key $KEYPATH/clientkey.pem -out $KEYPATH/client.csr -sha256
XY
Any State
Any Town
Any Organization
Any Unit
$HOST
postmaster@localhost


EOF

        # Create files required by the default_ca section of the standard openssl.cnf configuration file:
        cwd=`pwd`
        cd /tmp
        mkdir -p demoCA/newcerts
    	# Create and initialze the serial file.
        echo 01 >> demoCA/serial
    	# Create empty index.txt file.
        touch demoCA/index.txt

        cat << EOF | $OPENSSLCLI ca -md sha256 -days 3650 -notext -in $KEYPATH/client.csr -cert $CERTPATH/CAcrt.pem -keyfile $CERTPATH/CAkey.pem -out $KEYPATH/clientcrt.pem
y
y
EOF

    	# Create PKCS12 format file for Client cert/key
    	$OPENSSLCLI pkcs12 -name ha_client -export -in $KEYPATH/clientcrt.pem -inkey $KEYPATH/clientkey.pem -out $KEYPATH/ha_client.p12 -descert -password pass:ibmss01

        # Reset the environment
        rm -rf demoCA
        cd $cwd
    fi

    if [ ! -f $CERTPATH/CAcrt.pem ]
    then
        echo Error: CA certificate not generated
        exit 2
    fi

    if [ ! -f $KEYPATH/clientcrt.pem ]
    then
        echo Error: client certificate not generated
        exit 2
    fi
}

DeleteKeys()
{
    # Delete the CA certificate because (a) it will expire, and (b) additional security.
    rm -f $CERTPATH/CAkey.pem
    rm -f $CERTPATH/CA.csr  # remove in CreateKeys()?
    rm -f $CERTPATH/CAcrt.pem
    rm -f $CERTPATH/ibm.security.ha.p12

    # Delete the client certificate.
    rm -f $KEYPATH/clientkey.pem
    rm -f $KEYPATH/client.csr   # remove in CreateKeys()?
    rm -f $KEYPATH/clientcrt.pem
    rm -f $KEYPATH/ha_client.p12
}

ConfigureServer()
{
    if [ $1 = "et" ]; then
        # Explicit-trust
        /bin/echo "
SSLVerifyClient none
SSLCACertificateFile /etc/apache2/ssl.crt/CAcrt.pem
<Location \"/ha\">
    order deny,allow
    deny from all
    allow from all
    SSLVerifyClient require
    SSLVerifyDepth 1
    Satisfy Any
</Location>" >$APACHECONF

    elif [ $1 = "ca" ]; then

        /bin/echo "
<Directory \"/var/www/lmi/public/ca\">
    order deny,allow
    deny from all
    allow from all
    Satisfy Any
</Directory>" >$APACHECONF

        if [ ! -h ${APACHECAPATH}/CAcrt.pem -a -f ${CERTPATH}/CAcrt.pem ]; then
            ln -s ${CERTPATH}/CAcrt.pem ${APACHECAPATH}/CAcrt.pem
        fi
    elif [ $1 = "ha" ]; then

        /bin/echo "
SSLVerifyClient none
SSLCACertificateFile /etc/apache2/ssl.crt/CAcrt.pem
<Location \"/ha\">
    order deny,allow
    deny from all
    allow from all
    SSLVerifyClient require
    SSLVerifyDepth 1
    Satisfy Any
</Location>
<Directory \"/var/www/lmi/public/ca\">
    order deny,allow
    deny from all
    allow from all
    Satisfy Any
</Directory>" >$APACHECONF

        if [ ! -h ${APACHECAPATH}/CAcrt.pem -a -f ${CERTPATH}/CAcrt.pem ]; then
            ln -s ${CERTPATH}/CAcrt.pem ${APACHECAPATH}/CAcrt.pem
        fi
    fi
    sync

    # Restart the web server to read the new configuration
    # while preserving open connections.
    /sbin/service apache2-lmi graceful
}

ResetServer()
{
    if [ -s $APACHECONF ]
    then
        /bin/rm $APACHECONF
	sync

        # Restart the web server to read the new configuration
        # while preserving open connections.
        /sbin/service apache2-lmi graceful
    fi
}

#####################################################

export OPENSSL_FIPS=1
#getopt
case $1 in
    -k)
        if [ $# -gt 2 ]
        then
            Usage
            exit 1
        fi
        if [ $# -eq 2 ]
        then
       	    PASSWORD=$2
        fi
        CreateKeys
        ;;
    -d)
        if [ $# -ne 1 ]
        then
            Usage
            exit 1
        fi
        DeleteKeys
        ;;
    -c)
        if [ $# -ne 2 ]
        then
            Usage
            exit 1
        fi
        IPADDRESS=$2
        # Configure HA for first-time-trust setup
        ConfigureServer ca
        ;;
    -h)
        if [ $# -ne 2 ]
        then
            Usage
            exit 1
        fi
        IPADDRESS=$2
        # Configure HA for first-time-trust operation
        ConfigureServer ha
        ;;
    -m)
        if [ $# -ne 2 ]
        then
            Usage
            exit 1
        fi
        IPADDRESS=$2
        # Configure HA for explicit-trust (manual configuration)
        ConfigureServer et
        ;;
    -r)
        if [ $# -ne 1 ]
        then
            Usage
            exit 1
        fi
        ResetServer
        ;;
    *)
        Usage
        exit 1
        ;;
esac

exit 0

