class AlertLogModel
  require 'sqlite3'
  
  attr_reader :size, :oldest_date, :newest_date, :oldest_id, :newest_id, :modification_time, :database_path
  
  def initialize(database_path, filter=nil)
    @database_path = database_path
    @filter_where_clause, @filter_bind_vars = build_filter_clause(filter)
    @debug = 0
    describe_database
  end
  
  def find(offset, result_size)
    before = Time.now
    offset_where_clause = "ID between #{@oldest_id} and #{@newest_id} "
    attach_arr = ["attach database '#{Dbmap.issues}' as Issues"]
    
    query, bind_vars = construct_query("select a.ID as ID, a.IssueID as IssueID, Occurred, AlertName, AlertType, ProtocolType, ProtectionDomain, SourceIP, DestIP, SourcePort, DestPort, a.Priority as Priority, VulnStatus, VlanID, AVPData, CheckName, Description, IsAttack, IsAudit from AlertsView a", offset_where_clause, result_size, offset)

    db = SqliteSource.initialize_database(@database_path, true, attach_arr)
    results = SqliteSource.query_database(db, @database_path, query, *bind_vars)
    query_time = Time.now - before
    logIt("query time = #{query_time} seconds",1,'find','AlertLogModel')
    finalized_rows(results)
  end
  
  def modified?
    @modification_time != db_mtime
  end
  
  private
  def build_filter_clause(lmi_filter)
    where_string = ""
    bind_vars = []
    
    start_time = lmi_filter.start_time.blank? ? "" : FilterDate.convert_to_utc(FilterDate.parse_time(lmi_filter.start_time))
    end_time = lmi_filter.end_time.blank? ? "" : FilterDate.convert_to_utc(FilterDate.parse_time(lmi_filter.end_time))
    
    if lmi_filter.search_strings.has_key?(:filter) 
      lmi_filter.search_strings[:filter].each do |filter|
        unless filter[:text].blank?
          column = filter[:coltype]
          value = filter[:text]
          case
          when column == "risk"
            risk = case
            when value.downcase == "low" then "3"
            when value.downcase == "medium" then "2"
            when value.downcase == "high" then "1"
            else "3"
            end
            where_string = append_to_clause(where_string, "a.Priority = #{risk}")
          when column == "source_ip"
            where_string = append_to_clause(where_string, "SourceIP = ?")
            bind_vars << value
          when column == "source_port"
            where_string = append_to_clause(where_string, "SourcePort = ?")
            bind_vars << value
          when column == "target_ip"
            where_string = append_to_clause(where_string, "DestIP = ?")
            bind_vars << value
          when column == "target_port"
            where_string = append_to_clause(where_string, "DestPort = ?")
            bind_vars << value
          when column == "protection_domain"
            where_string = append_to_clause(where_string, "lower(ProtectionDomain) = ?")
            bind_vars << value.downcase
          when column == "vlanid"
            where_string = append_to_clause(where_string, "VlanID = ?")
            bind_vars << value
          when column == "protocoltype"
            where_string = append_to_clause(where_string, "ProtocolType = ?")
            bind_vars << value
          when (column == "status" and ( value == "Detected" or value == "Blocked" or value == "Quarantined" or value == "Simulated Block"))
            status = case
            when value == "Detected" then "123"
            when value == "Blocked" then "126"
            when value == "Quarantined" then "125"
            when value == "Simulated Block" then "124"
            end
            where_string = append_to_clause(where_string, "VulnStatus = '#{status}'")
          when column == "issueid"
            where_string = append_to_clause(where_string, "a.IssueID = ?")
            bind_vars << value
          when column == "event_name"
            where_string = append_to_clause(where_string, "(i.CheckName like ? escape '!' or AlertName like ? escape '!')")
            escaped_value = value.gsub('!', '!!').gsub('%', '!%').gsub('_', '!_')
            bind_vars << "%#{escaped_value}%" << "%#{escaped_value}%"
          when column == "event_type"
            where_string = append_to_clause(where_string, "a.AlertType = ?")
            bind_vars << value
          end
        end
      end
    end

    where_string = append_to_clause(where_string, "Occurred >= '#{start_time}'") unless start_time.blank?
    where_string = append_to_clause(where_string, "Occurred <= '#{end_time}'") unless end_time.blank?
    return where_string, bind_vars
  end

  def add_issues_join(query, where_clause)
    s = ''
    unless where_clause.match(/CheckName/).nil? and query.match(/CheckName/).nil?
      s = " LEFT OUTER JOIN Issues.IssuesView i ON (CASE WHEN a.AlertType IN ('Attack', 'Audit') THEN a.IssueID END) = i.IssueId "
    end
    s
  end
  
  def construct_query(query, extra_where_clause="", result_size=nil, offset=0)
    where_clause, bind_vars = construct_where_clause(extra_where_clause)
    query << add_issues_join(query, where_clause)
    query << where_clause
    query << " order by a.ID"
    if !result_size.nil? 
      query << " limit #{offset},#{result_size}"
    end
    log_query(query, bind_vars)
    return query, bind_vars
  end
  
  def construct_where_clause(clause)
    where_clause = ""
    bind_vars = []
    where_clause = append_to_clause(where_clause, clause)
    where_clause = append_to_clause(where_clause, @filter_where_clause)
    bind_vars += @filter_bind_vars
    if where_clause != ""
      where_clause = " where " + where_clause
    end
    return where_clause, bind_vars
  end
  
  def append_to_clause(clause, param)
    if param != ""
      if clause == ""
        clause = param
      else
        clause += " and " + param
      end
    end
    clause
  end
  
  def log_query(query, bind_vars)
    logIt("#{Time.now} DB: #{@database_path}",1,'log_query','AlertLogModel')
    logIt("Query: #{query}",1,'log_query','AlertLogModel')
    logIt("Vars: #{bind_vars * ","}",1,'log_query','AlertLogModel')
  end
  
  def describe_database
    logIt("Starting for #{@database_path}...",1,'describe_database','AlertLogModel')
    before = Time.now
    query_str = "select min(ID) as min_id, max(ID) as max_id, count(ID) as total from AlertsView a"
    attach_arr = ["attach database '#{Dbmap.issues}' as Issues"]      
    query, bind_vars = construct_query(query_str)
    db = SqliteSource.initialize_database(@database_path, true, attach_arr)
    first_row = SqliteSource.get_first_row(db, @database_path, query, *bind_vars)
    if first_row.nil?   # failsafe: get_first_row may return nil
      @size = @oldest_id = @newest_id = 0
    else
      @size = first_row['total'].to_i
      @oldest_id = first_row['min_id'].to_i
      @newest_id = first_row['max_id'].to_i
    end
   
    date_query1 = "select Occurred from AlertsView where ID = '#{@newest_id}'"
    date_query2 = "select Occurred from AlertsView where ID = '#{@oldest_id}'"
       
    db = SqliteSource.initialize_database(@database_path, true, attach_arr)
    @newest_date = SqliteSource.get_first_value(db, @database_path, date_query1)
 
    db = SqliteSource.initialize_database(@database_path, true, attach_arr)
    @oldest_date = SqliteSource.get_first_value(db, @database_path, date_query2)
      
    query_time = Time.now - before
    logIt("query time = #{query_time} seconds",1,'describe_database','AlertLogModel')
    logIt("oldest/minID: #{@oldest_id} || newest/maxID: #{@newest_id} || TotalRows: #{@size}",0,'describe_database','AlertLogModel',false)
    logIt("oldestDate: #{@oldest_date}   ||   newestDate: #{@newest_date}",0,'describe_database','AlertLogModel',false)
    logIt("Finished...",1,'describe_database','AlertLogModel')
    logIt("",1,"","") #just spit out a blank line for readability in logs
    @modification_time = db_mtime
  end
  
  def db_mtime
    db_file = File.new(@database_path)
    begin
      changed_time = db_file.mtime
    ensure
      db_file.close unless db_file.nil?
    end
    changed_time
  end
 
  def finalized_rows(rows)
    rows.each do | row |
      row = modified_date(row)
      row = modified_priority(row)
      row = modified_status(row)
      row = modified_avp(row)
      row = modified_name(row)      
    end
    rows
  end
  
  def modified_date(row)
    if row['Occurred'].nil?
      row['Occurred'] = ""
      return row
    end
    row['Occurred'] = FilterDate.convert_to_local(row['Occurred'])
    row
  end
  
  def modified_name(row)
    if row['AlertType'] !~ /^attack$/i
      row["OutName"] = row["AlertName"]
    else
      row["OutName"] = "#{row['CheckName']}"
    end
    row
  end

  def modified_priority(row)
    priority_code = row['Priority']
    priority_display = case
    when priority_code == "3" then "Low"
    when priority_code == "2" then "Medium"
    when priority_code == "1" then "High"
    else "Unknown"
    end
    row['Priority'] = priority_display
    row
  end
  
  def modified_status(row)
    status_code = row['VulnStatus']
    status_display = case
      when status_code == "126" then "Blocked"
      when status_code == "125" then "Quarantined"
      when status_code == "124" then "Simulated Block"
      else "Detected"
    end
    row['VulnStatus'] = status_display
    row           
  end
  
  def modified_avp(row)
    if row['AVPData'].nil?
      row['AVPData'] = ''
      row['ResponseList'] = ''
      row['Message'] = ''
    end
    avp_string = row['AVPData']
    new_avp_string = ""
   
    blocks = avp_string.split(";")
    blocks.each do |block|
      contents = block.split(":", 2)
      id = contents[0]
      value = contents[1]
      case 
      when (id == "ResponseList" or id == "Message") then row[id] = value
      else 
        new_avp_string += block + ";"
      end
    end
    row['AVPData'] = new_avp_string
    row
  end

  def logIt(msg, overrideDebug=0, func='', filename='', hi_lite=true)
    if (!msg.nil? and ((!@debug.zero?) or (!overrideDebug.zero?)))
      wrap_head = ''
      wrap_tail = ''
      output = ''
      if (!filename.empty?)
        output += "[#{filename}] "
      end
      if(!func.empty?)
        output += "<#{func}> "
      end
      output << msg 
      if hi_lite
        wrap_head = "\e[1;34m"
        wrap_tail = "\e[0m"
      end
      output = wrap_head << ' ' << output << ' ' << wrap_tail
      RAILS_DEFAULT_LOGGER.info(output)
    end
  end

end
